#//-----------------------------------------------
#//TUSB-KINSxx用ドライバアクセス用API
#//Python 用モジュール
#//
#//2024_04_09　株式会社タートル工業
#//Copyright (C) 2024 Turtle Industry Co.,Ltd.
#//-----------------------------------------------

from ctypes import *

#Device DLL Loader ( cdecl )
tusbkinsdll = cdll.LoadLibrary("TUSBKINS.DLL")

tusbkinsdll.TUSBKINS_Device_Open.restype = c_short
tusbkinsdll.TUSBKINS_Device_Open.argtypes = (c_short,)
tusbkinsdll.TUSBKINS_Device_Close.restype = None
tusbkinsdll.TUSBKINS_Device_Close.argtypes = (c_short,)
tusbkinsdll.TUSBKINS_Write.restype = c_short
tusbkinsdll.TUSBKINS_Write.argtypes = (c_short,c_uint)
tusbkinsdll.TUSBKINS_Read.restype = c_short
tusbkinsdll.TUSBKINS_Read.argtypes = (c_short,POINTER(c_uint),POINTER(c_uint))
tusbkinsdll.TUSBKINS_EdgeSet.restype = c_short
tusbkinsdll.TUSBKINS_EdgeSet.argtypes = (c_short,c_uint,c_ubyte)
tusbkinsdll.TUSBKINS_EdgeRead.restype = c_short
tusbkinsdll.TUSBKINS_EdgeRead.argtypes = (c_short,POINTER(c_uint))
tusbkinsdll.TUSBKINS_Type.restype = c_short
tusbkinsdll.TUSBKINS_Type.argtypes = (c_short,POINTER(c_ubyte))

#######  support functions (don not call directry from user codes)

def isint(n):
    return type(n) is int

def IsInRange(Val,Min,Max):
    if(isint(Val) == False ):   return False
    elif(Val < Min):            return False
    elif(Val > Max):            return False
    else:                       return True

def IsValidId(DevId):
    return IsInRange(DevId,0,15)

# def functions ------------------------------------
def TUSBKINS_Device_Open(DevId):
    """Open Device Connection
    
    Arguments:
    DevId           ----    Device ID 0~15

    Returns:
    int             ----    Result code
    """
    if(not IsValidId(DevId)): return 8 # Invalid parameter
    return tusbkinsdll.TUSBKINS_Device_Open(DevId)

def TUSBKINS_Device_Close(DevId):
    """Close Device Connection
    
    Arguments:
    DevId           ----    Device ID 0~15

    Returns:
    int             ----    Result code
    """
    if(not IsValidId(DevId)): return # Invalid parameter
    tusbkinsdll.TUSBKINS_Device_Close(DevId)

def TUSBKINS_Write(DevId,Data):
    """Write Output Resisters
    
    Arguments:
    DevId           ----    Device ID 0~15
    Data            ----    Output data bits

    Returns;
    int             ----    Result code
    """
    if(not IsValidId(DevId)): return 8 # Invalid parameter
    if(not IsInRange(Data,0,65535)): return 8 # Invalid parameter
    return tusbkinsdll.TUSBKINS_Write(DevId,Data)

def TUSBKINS_Read(DevId):
    """Read Input Resister
    
    Arguments:
    DevId           ----    Device ID 0~15
    
    Returns:
    int             ----    Result code
    int             ----    Input Data
    int             ----    Output Status
    """
    if(not IsValidId(DevId)): return 8,None,None # Invalid parameter
    InData = c_uint()
    OUTStatus = c_uint()
    ret =  tusbkinsdll.TUSBKINS_Read(DevId,byref(InData),byref(OUTStatus))
    if ret == 0:
        return 0,InData.value,OUTStatus.value
    else:
        return ret,None,None
    
def TUSBKINS_EdgeSet(DevId,Data,Time):
    """Write Output Resisters
    
    Arguments:
    DevId           ----    Device ID 0~15
    Data            ----    Edge Direction bits
    Time            ----    Detection time 0~255

    Returns;
    int             ----    Result code
    """
    if(not IsValidId(DevId)): return 8 # Invalid parameter
    if(not IsInRange(Data,0,65535)): return 8 # Invalid parameter
    if(not IsInRange(Time,0,255)): return 8 # Invalid parameter
    return tusbkinsdll.TUSBKINS_EdgeSet(DevId,Data,Time)

def TUSBKINS_EdgeRead(DevId):
    """Read Input Resister
    
    Arguments:
    DevId           ----    Device ID 0~15
    
    Returns:
    int             ----    Result code
    int             ----    Detected Edge Data bits
    """
    if(not IsValidId(DevId)): return 8,None # Invalid parameter
    Data = c_uint()
    ret =  tusbkinsdll.TUSBKINS_EdgeRead(DevId,byref(Data))
    if ret == 0:
        return 0,Data.value
    else:
        return ret,None
    
def TUSBKINS_Type(DevId):
    """Read Input Resister
    
    Arguments:
    DevId           ----    Device ID 0~15
    
    Returns:
    int             ----    Result code
    int             ----    Type code
    """
    if(not IsValidId(DevId)): return 8,None # Invalid parameter
    devType = c_ubyte()
    ret =  tusbkinsdll.TUSBKINS_Type(DevId,byref(devType))
    if ret == 0:
        return 0,devType.value
    else:
        return ret,None
    
#戻り値用エラーメッセージ(オープン関数)
ErrStrDic = {
            0:"正常終了しました",
            2:"ドライバが初期化されていません",
            3:"すでにデバイスはオープンされています",
            4:"接続されている台数が多すぎます",
            5:"オープンできませんでした",
            6:"デバイスがみつかりません",
            8:"パラメータエラーです",
            9:"USB通信エラーです",
            99:"その他のエラーです"
}
def GetErrMessage(retcode):
    if( retcode in ErrStrDic):
        return ErrStrDic[retcode]
    else:
        return "不明なエラーです"
